@@ -15,6 +15,7 @@
 
 #include <gtk-utils.hpp>
 #include <gtk-layer-shell.h>
+#include <glib-unix.h>
 
 #include "background.hpp"
 
@@ -35,6 +36,12 @@ void BackgroundDrawingArea::show_image(Glib::RefPtr<Gdk::Pixbuf> image,
 
     to_image.x = offset_x / this->get_scale_factor();
     to_image.y = offset_y / this->get_scale_factor();
+
+    fade = {
+        fade_duration,
+        wf::animation::smoothing::linear
+    };
+
     fade.animate(from_image.source ? 0.0 : 1.0, 1.0);
 
     Glib::signal_idle().connect_once([=] ()
@@ -100,7 +107,7 @@ Glib::RefPtr<Gdk::Pixbuf> WayfireBackground::create_from_file_safe(std::string p
     return pbuf;
 }
 
-bool WayfireBackground::change_background(int timer)
+bool WayfireBackground::change_background()
 {
     Glib::RefPtr<Gdk::Pixbuf> pbuf;
     std::string path;
@@ -120,15 +127,21 @@ bool WayfireBackground::load_images_from_dir(std::string path)
     wordexp_t exp;
 
     /* Expand path */
-    wordexp(path.c_str(), &exp, 0);
-    if (!exp.we_wordv)
+    if (wordexp(path.c_str(), &exp, 0))
     {
         return false;
     }
 
+    if (!exp.we_wordc)
+    {
+        wordfree(&exp);
+        return false;
+    }
+
     auto dir = opendir(exp.we_wordv[0]);
     if (!dir)
     {
+        wordfree(&exp);
         return false;
     }
 
@@ -158,6 +171,8 @@ bool WayfireBackground::load_images_from_dir(std::string path)
         }
     }
 
+    wordfree(&exp);
+
     if (background_randomize && images.size())
     {
         std::random_device random_device;
@@ -248,8 +263,8 @@ void WayfireBackground::reset_cycle_timeout()
     change_bg_conn.disconnect();
     if (images.size())
     {
-        change_bg_conn = Glib::signal_timeout().connect(sigc::bind(sigc::mem_fun(
-            this, &WayfireBackground::change_background), 0), cycle_timeout);
+        change_bg_conn = Glib::signal_timeout().connect(sigc::mem_fun(
+            this, &WayfireBackground::change_background), cycle_timeout);
     }
 }
 
@@ -306,6 +321,11 @@ WayfireBackground::WayfireBackground(WayfireShellApp *app, WayfireOutput *output
     });
 }
 
+WayfireBackground::~WayfireBackground()
+{
+    reset_background();
+}
+
 class WayfireBackgroundApp : public WayfireShellApp
 {
     std::map<WayfireOutput*, std::unique_ptr<WayfireBackground>> backgrounds;
@@ -316,6 +336,7 @@ class WayfireBackgroundApp : public WayfireShellApp
     {
         WayfireShellApp::instance =
             std::make_unique<WayfireBackgroundApp>(argc, argv);
+        g_unix_signal_add(SIGUSR1, sigusr1_handler, (void*)instance.get());
         instance->run();
     }
 
@@ -329,6 +350,16 @@ class WayfireBackgroundApp : public WayfireShellApp
     {
         backgrounds.erase(output);
     }
+
+    static gboolean sigusr1_handler(void *instance)
+    {
+        for (const auto& [_, bg] : ((WayfireBackgroundApp*)instance)->backgrounds)
+        {
+            bg->change_background();
+        }
+
+        return TRUE;
+    }
 };
 
 int main(int argc, char **argv)
