@@ -11,7 +11,6 @@
 #include <gdk/gdkwayland.h>
 #include <cmath>
 
-
 #include "toplevel.hpp"
 #include "gtk-utils.hpp"
 #include "panel.hpp"
@@ -44,6 +43,8 @@ class WayfireToplevel::impl
 
     Glib::ustring app_id, title;
 
+    WfOption<bool> middle_click_close{"panel/middle_click_close"};
+
   public:
     WayfireWindowList *window_list;
 
@@ -182,14 +183,20 @@ class WayfireToplevel::impl
 
     bool on_button_press_event(GdkEventButton *event)
     {
-        if ((event->type == GDK_BUTTON_PRESS) && (event->button == 3))
+        if (event->type == GDK_BUTTON_PRESS)
         {
-            menu.popup(event->button, event->time);
-            return true; // It has been handled.
-        } else
-        {
-            return false;
+            if (event->button == 3)
+            {
+                menu.popup(event->button, event->time);
+                return true; // It has been handled.
+            } else if ((event->button == 2) && middle_click_close)
+            {
+                zwlr_foreign_toplevel_handle_v1_close(handle);
+                return true;
+            }
         }
+
+        return false;
     }
 
     void on_menu_minimize()
@@ -291,35 +298,26 @@ class WayfireToplevel::impl
 
     void set_app_id(std::string app_id)
     {
+        WfOption<int> minimal_panel_height{"panel/minimal_height"};
         this->app_id = app_id;
         IconProvider::set_image_from_icon(image, app_id,
-            24, button.get_scale_factor());
+            std::min(int(minimal_panel_height), 24), button.get_scale_factor());
     }
 
     void send_rectangle_hint()
     {
         Gtk::Widget *widget = &this->button;
-
-        int x = 0, y = 0;
-        int width = button.get_allocated_width();
-        int height = button.get_allocated_height();
-
-        while (widget)
+        auto panel = WayfirePanelApp::get().panel_for_wl_output(window_list->output->wo);
+        if (panel)
         {
-            x += widget->get_allocation().get_x();
-            y += widget->get_allocation().get_y();
-            widget = widget->get_parent();
-        }
+            int x, y;
+            widget->translate_coordinates(panel->get_window(), 0, 0, x, y);
 
-        auto panel =
-            WayfirePanelApp::get().panel_for_wl_output(window_list->output->wo);
-        if (!panel)
-        {
-            return;
+            int width  = button.get_allocated_width();
+            int height = button.get_allocated_height();
+            zwlr_foreign_toplevel_handle_v1_set_rectangle(handle,
+                panel->get_wl_surface(), x, y, width, height);
         }
-
-        zwlr_foreign_toplevel_handle_v1_set_rectangle(handle,
-            panel->get_wl_surface(), x, y, width, height);
     }
 
     int32_t max_width = 0;
@@ -727,7 +725,7 @@ void set_image_from_icon(Gtk::Image& image,
         {
             /* Perhaps no desktop app info, but we might still be able to
              * get an icon directly from the icon theme */
-            if (Gtk::IconTheme::get_default()->lookup_icon(app_id, 24))
+            if (Gtk::IconTheme::get_default()->lookup_icon(app_id, size))
             {
                 icon_name = app_id;
             }
